@@ -1,12 +1,10 @@
 package HTTP::Headers;
 
-# $Id: Headers.pm,v 1.62 2004/11/12 15:38:38 gisle Exp $
-
 use strict;
 use Carp ();
 
 use vars qw($VERSION $TRANSLATE_UNDERSCORE);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.62 $ =~ /(\d+)\.(\d+)/);
+$VERSION = "5.822";
 
 # The $TRANSLATE_UNDERSCORE variable controls whether '_' can be used
 # as a replacement for '-' in header field names.
@@ -19,34 +17,34 @@ $TRANSLATE_UNDERSCORE = 1 unless defined $TRANSLATE_UNDERSCORE;
 #    - Entity-Headers
 
 my @general_headers = qw(
-   Cache-Control Connection Date Pragma Trailer Transfer-Encoding Upgrade
-   Via Warning
+    Cache-Control Connection Date Pragma Trailer Transfer-Encoding Upgrade
+    Via Warning
 );
 
 my @request_headers = qw(
-   Accept Accept-Charset Accept-Encoding Accept-Language
-   Authorization Expect From Host
-   If-Match If-Modified-Since If-None-Match If-Range If-Unmodified-Since
-   Max-Forwards Proxy-Authorization Range Referer TE User-Agent
+    Accept Accept-Charset Accept-Encoding Accept-Language
+    Authorization Expect From Host
+    If-Match If-Modified-Since If-None-Match If-Range If-Unmodified-Since
+    Max-Forwards Proxy-Authorization Range Referer TE User-Agent
 );
 
 my @response_headers = qw(
-   Accept-Ranges Age ETag Location Proxy-Authenticate Retry-After Server
-   Vary WWW-Authenticate
+    Accept-Ranges Age ETag Location Proxy-Authenticate Retry-After Server
+    Vary WWW-Authenticate
 );
 
 my @entity_headers = qw(
-   Allow Content-Encoding Content-Language Content-Length Content-Location
-   Content-MD5 Content-Range Content-Type Expires Last-Modified
+    Allow Content-Encoding Content-Language Content-Length Content-Location
+    Content-MD5 Content-Range Content-Type Expires Last-Modified
 );
 
 my %entity_header = map { lc($_) => 1 } @entity_headers;
 
 my @header_order = (
-   @general_headers,
-   @request_headers,
-   @response_headers,
-   @entity_headers,
+    @general_headers,
+    @request_headers,
+    @response_headers,
+    @entity_headers,
 );
 
 # Make alternative representations of @header_order.  This is used
@@ -79,8 +77,11 @@ sub header
     my $self = shift;
     Carp::croak('Usage: $h->header($field, ...)') unless @_;
     my(@old);
-    while (my($field, $val) = splice(@_, 0, 2)) {
-	@old = $self->_header($field, $val);
+    my %seen;
+    while (@_) {
+	my $field = shift;
+        my $op = @_ ? ($seen{lc($field)}++ ? 'PUSH' : 'SET') : 'GET';
+	@old = $self->_header($field, shift, $op);
     }
     return @old if wantarray;
     return $old[0] if @old <= 1;
@@ -96,8 +97,11 @@ sub clear
 
 sub push_header
 {
-    Carp::croak('Usage: $h->push_header($field, $val)') if @_ != 3;
-    shift->_header(@_, 'PUSH');
+    my $self = shift;
+    return $self->_header(@_, 'PUSH_H') if @_ == 2;
+    while (@_) {
+	$self->_header(splice(@_, 0, 2), 'PUSH_H');
+    }
 }
 
 
@@ -142,9 +146,6 @@ sub _header
 {
     my($self, $field, $val, $op) = @_;
 
-    # $push is only used interally sub push_header
-    Carp::croak('Need a field name') unless length($field);
-
     unless ($field =~ /^:/) {
 	$field =~ tr/_/-/ if $TRANSLATE_UNDERSCORE;
 	my $old = $field;
@@ -156,20 +157,40 @@ sub _header
 	}
     }
 
+    $op ||= defined($val) ? 'SET' : 'GET';
+    if ($op eq 'PUSH_H') {
+	# Like PUSH but where we don't care about the return value
+	if (exists $self->{$field}) {
+	    my $h = $self->{$field};
+	    if (ref($h) eq 'ARRAY') {
+		push(@$h, ref($val) eq "ARRAY" ? @$val : $val);
+	    }
+	    else {
+		$self->{$field} = [$h, ref($val) eq "ARRAY" ? @$val : $val]
+	    }
+	    return;
+	}
+	$self->{$field} = $val;
+	return;
+    }
+
     my $h = $self->{$field};
     my @old = ref($h) eq 'ARRAY' ? @$h : (defined($h) ? ($h) : ());
 
-    $op ||= "";
-    $val = undef if $op eq 'INIT' && @old;
-    if (defined($val)) {
-	my @new = ($op eq 'PUSH') ? @old : ();
-	if (ref($val) ne 'ARRAY') {
-	    push(@new, $val);
+    unless ($op eq 'GET' || ($op eq 'INIT' && @old)) {
+	if (defined($val)) {
+	    my @new = ($op eq 'PUSH') ? @old : ();
+	    if (ref($val) ne 'ARRAY') {
+		push(@new, $val);
+	    }
+	    else {
+		push(@new, @$val);
+	    }
+	    $self->{$field} = @new > 1 ? \@new : $new[0];
 	}
-	else {
-	    push(@new, @$val);
+	elsif ($op ne 'PUSH') {
+	    delete $self->{$field};
 	}
-	$self->{$field} = @new > 1 ? \@new : $new[0];
     }
     @old;
 }
@@ -236,12 +257,15 @@ sub as_string
 }
 
 
-sub clone
-{
-    my $self = shift;
-    my $clone = new HTTP::Headers;
-    $self->scan(sub { $clone->push_header(@_);} );
-    $clone;
+if (eval { require Storable; 1 }) {
+    *clone = \&Storable::dclone;
+} else {
+    *clone = sub {
+	my $self = shift;
+	my $clone = new HTTP::Headers;
+	$self->scan(sub { $clone->push_header(@_);} );
+	$clone;
+    };
 }
 
 
@@ -253,6 +277,7 @@ sub _date_header
     if (defined $time) {
 	$self->_header($header, HTTP::Date::time2str($time));
     }
+    $old =~ s/;.*// if defined($old);
     HTTP::Date::str2time($old);
 }
 
@@ -275,14 +300,36 @@ sub client_date         { shift->_date_header('Client-Date',         @_); }
 #sub retry_after       { shift->_date_header('Retry-After',       @_); }
 
 sub content_type      {
-  my $ct = (shift->_header('Content-Type', @_))[0];
-  return '' unless defined($ct) && length($ct);
-  my @ct = split(/;\s*/, $ct, 2);
-  for ($ct[0]) {
-      s/\s+//g;
-      $_ = lc($_);
-  }
-  wantarray ? @ct : $ct[0];
+    my $self = shift;
+    my $ct = $self->{'content-type'};
+    $self->{'content-type'} = shift if @_;
+    $ct = $ct->[0] if ref($ct) eq 'ARRAY';
+    return '' unless defined($ct) && length($ct);
+    my @ct = split(/;\s*/, $ct, 2);
+    for ($ct[0]) {
+	s/\s+//g;
+	$_ = lc($_);
+    }
+    wantarray ? @ct : $ct[0];
+}
+
+sub content_is_html {
+    my $self = shift;
+    return $self->content_type eq 'text/html' || $self->content_is_xhtml;
+}
+
+sub content_is_xhtml {
+    my $ct = shift->content_type;
+    return $ct eq "application/xhtml+xml" ||
+           $ct eq "application/vnd.wap.xhtml+xml";
+}
+
+sub content_is_xml {
+    my $ct = shift->content_type;
+    return 1 if $ct eq "text/xml";
+    return 1 if $ct eq "application/xml";
+    return 1 if $ct =~ /\+xml$/;
+    return 0;
 }
 
 sub referer           {
@@ -394,7 +441,9 @@ Returns a copy of this C<HTTP::Headers> object.
 
 =item $h->header( $field )
 
-=item $h->header( $field => $value, ... )
+=item $h->header( $field => $value )
+
+=item $h->header( $f1 => $v1, $f2 => $v2, ... )
 
 Get or set the value of one or more header fields.  The header field
 name ($field) is not case sensitive.  To make the life easier for perl
@@ -405,8 +454,9 @@ The header() method accepts multiple ($field => $value) pairs, which
 means that you can update several fields with a single invocation.
 
 The $value argument may be a plain string or a reference to an array
-of strings for a multi-valued field. If the $value is undefined or not
-given, then that header field will remain unchanged.
+of strings for a multi-valued field. If the $value is provided as
+C<undef> then the field is removed.  If the $value is not given, then
+that header field will remain unchanged.
 
 The old value (or values) of the last of the header fields is returned.
 If no such field exists C<undef> will be returned.
@@ -430,6 +480,8 @@ Examples:
 
 =item $h->push_header( $field => $value )
 
+=item $h->push_header( $f1 => $v1, $f2 => $v2, ... )
+
 Add a new field value for the specified header field.  Previous values
 for the same field are retained.
 
@@ -521,7 +573,7 @@ values will be substituted with this line ending sequence.
 =head1 CONVENIENCE METHODS
 
 The most frequently used headers can also be accessed through the
-following convenience methods.  These methods can both be used to read
+following convenience Methods.  Most of these methods can both be used to read
 and to set the value of a header.  The header value is set if you pass
 an argument to the method.  The old header value is always returned.
 If the given header did not exist then C<undef> is returned.
@@ -583,6 +635,22 @@ string is returned.  This makes it safe to do the following:
      ...
   }
 
+=item $h->content_is_html
+
+Returns TRUE if the Content-Type header field indicate that the
+content is some kind of HTML (including XHTML).  This method can't be
+used to set Content-Type.
+
+=item $h->content_is_xhtml
+
+Returns TRUE if the Content-Type header field indicate that the
+content is XHTML.  This method can't be used to set Content-Type.
+
+=item $h->content_is_xml
+
+Returns TRUE if the Content-Type header field indicate that the
+content is XML.  This method can't be used to set Content-Type.
+
 =item $h->content_encoding
 
 The Content-Encoding header field is used as a modifier to the
@@ -612,7 +680,7 @@ standard.>
 This header field is used in request messages and contains information
 about the user agent originating the request.  I<E.g.>:
 
-  $h->user_agent('Mozilla/1.2');
+  $h->user_agent('Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.0)');
 
 =item $h->server
 
@@ -708,19 +776,9 @@ These field names are returned with the ':' intact for
 $h->header_field_names and the $h->scan callback, but the colons do
 not show in $h->as_string.
 
-=head1 BUGS
-
-In the argument list to the constructor or header() method, the same
-field name should not occur multiple times.  The result of doing so,
-it that only the last of these fields will be present in the header
-after the call.  All values ought to be kept.
-
-Passing a value of C<undef> to header() or any of the convenience
-methods, does not delete that field.  It ought to do that.
-
 =head1 COPYRIGHT
 
-Copyright 1995-2004 Gisle Aas.
+Copyright 1995-2005 Gisle Aas.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
